<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class CardtrabController extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }
    
    public function getCustos(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $conn = $this->entityManager->getConnection();
            $sql = "SELECT CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB, 'ALIMENTAR') AS ALIMENTAR,
                           CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB, 'DESCARTAVEL') AS DESCARTAVEL,
                           CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB, 'LIMPEZA') AS LIMPEZA,
                           CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB, 'EVENTO') AS EVENTO,
                           CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB) AS TOTAL,
                           PROD_CUSTZERO_CARDTRAB(C.NRCARDAPIOTRAB) AS TEM_PRODCUSTOZERO
                      FROM CARDAPIOTRAB C
                     WHERE C.NRCARDAPIOTRAB = :P_NRCARDAPIOTRAB";
            $params = [
                "P_NRCARDAPIOTRAB" => $row["NRCARDAPIOTRAB"]
            ];
            $query = $conn->prepare($sql);
        	$query->execute($params);
        	$custos = $query->fetch();
        	if(!$custos)
        	   $custos = ["NAO_POSSUI" => true];
        	$response->addDataSet(new DataSet('custos', $custos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getProdSemCusto(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $conn = $this->entityManager->getConnection();
            $sql = "SELECT DISTINCT PRA.CDPRATO, PRA.NMPRATO, PRO.CDARVPROD, PRO.NMPRODUTO
                      FROM CARDAPIOTRAB C
                           INNER JOIN COMPCARDTRAB CO ON CO.NRCARDAPIOTRAB = C.NRCARDAPIOTRAB
                           INNER JOIN RECEPRATCARDTRAB R ON R.NRCARDAPIOTRAB = CO.NRCARDAPIOTRAB AND
                                                            R.CDFILAUXPRAT = CO.CDFILAUXPRAT AND
                                                            R.CDPRATO = CO.CDPRATO
                           LEFT JOIN PRODUTO PRO ON PRO.CDPRODUTO = R.CDPRODUTO
                           LEFT JOIN PRATO PRA ON PRA.CDFILAUXPRAT = R.CDFILAUXPRAT AND
                                                  PRA.CDPRATO = R.CDPRATO
                     WHERE C.NRCARDAPIOTRAB = :P_NRCARDAPIOTRAB
                       AND CUSTO_PROD_CARDTRAB(R.NRCARDAPIOTRAB, R.CDPRODUTO) <= 0";
            $params = [
                "P_NRCARDAPIOTRAB" => $row["NRCARDAPIOTRAB"]
            ];
            $query = $conn->prepare($sql);
        	$query->execute($params);
        	$produtosSemCusto = $query->fetchAll();
        	$response->addDataSet(new DataSet('produtosSemCusto', $produtosSemCusto));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getCustosEvento(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $conn = $this->entityManager->getConnection();
            $sql = "SELECT CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB, 'ANTESEVENTO') AS ANTESEVENTO,
                           (SELECT COUNT(1) FROM EVENTOCARDTRAB E WHERE E.NRCARDAPIOTRAB = C.NRCARDAPIOTRAB) AS QTEVENTO,
                           CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB, 'EVENTO') AS EVENTO,
                           CUSTO_PERC_CARDTRAB(C.NRCARDAPIOTRAB) AS TOTAL
                      FROM CARDAPIOTRAB C
                     WHERE C.NRCARDAPIOTRAB = :NRCARDAPIOTRAB";
            $params = [
                "NRCARDAPIOTRAB" => $row["NRCARDAPIOTRAB"]
            ];
            $query = $conn->prepare($sql);
        	$query->execute($params);
        	$custosEvento = $query->fetch();
        	if(!$custosEvento)
        	   $custosEvento = ["NAO_POSSUI" => true];
        	$response->addDataSet(new DataSet('custosEvento', $custosEvento));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}