<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Tco01002ConfiguracaoCardapiobase extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;
    protected $elaboracaocardapioservice;

    public function __construct($crudService, $parameterBag, $environment, $entityManager, $elaboracaocardapioservice) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->elaboracaocardapioservice = $elaboracaocardapioservice;
    }
    
    protected $DELETE_CONFIGCARDBASE = "DELETE FROM CONFIGCARDBASE
                                         WHERE NRCONFIGCARDBASE = :NRCONFIGCARDBASE 
                                           AND NRORG = :NRORG";
                                           
    protected $DELETE_ITEMADESAOCBASE = "DELETE FROM ITEMADESAOCBASE
                                          WHERE NRCONFIGCARDBASE = :NRCONFIGCARDBASE 
                                            AND NRORG = :NRORG";
                                            
    protected $DELETE_ITPARTSEGCARDBASE = "DELETE FROM ITPARTSEGCARDBASE
                                            WHERE NRCONFIGCARDBASE = :NRCONFIGCARDBASE
                                              AND NRORG = :NRORG";
    
    protected $DELETE_DESCARTCARDBASE = "DELETE FROM DESCARTCARDBASE
                                          WHERE NRCONFIGCARDBASE = :NRCONFIGCARDBASE
                                            AND NRORG = :NRORG";
    
    protected $DELETE_LIMPEZACARDBASE = "DELETE FROM LIMPEZACARDBASE
                                          WHERE NRCONFIGCARDBASE = :NRCONFIGCARDBASE
                                            AND NRORG = :NRORG";
    
    protected $UPDATE_AMOSTRACARDBASEQUENTE = "UPDATE AMOSTRACARDBASE
                                                  SET NRCONFIGCARDBASEQ = NULL
                                                WHERE NRCONFIGCARDBASEQ = :NRCONFIGCARDBASE
                                                  AND NRORG = :NRORG";
    
    protected $UPDATE_AMOSTRACARDBASEFRIA = "UPDATE AMOSTRACARDBASE
                                                SET NRCONFIGCARDBASEF = NULL
                                              WHERE NRCONFIGCARDBASEF = :NRCONFIGCARDBASE
                                                AND NRORG = :NRORG";
    
    public function salvaConfiguracaoCardapioBase(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $rowsToUpdate = array();
        
        if($row["__is_new"]) {
            
            $row["NRORDEM"] = $this->retornaProximaOrdem($row);
            
        } else if($row["ORDUP"]) {
            $rowAfetada = $this->retornaConfigAfetada($row, $row["NRORDEM"]);
            
            if($rowAfetada){
                $nrordemRowAfetada = $rowAfetada["NRORDEM"];
                $nrordemRow = $row["NRORDEM"];
                
                $row["NRORDEM"] = $nrordemRowAfetada;
                $rowAfetada["NRORDEM"] = $nrordemRow;
                
                $rowAfetada["__is_new"] = false;
                $rowAfetada["changed"] = true;
                array_push($rowsToUpdate, $row, $rowAfetada);
            } else {
                array_push($rowsToUpdate, $row);
            }
            
        } else if($row["ORDDW"]) {
            
            $rowAfetada = $this->retornaConfigAfetada($row, $row["NRORDEM"]);
            
            if($rowAfetada){
                $nrordemRowAfetada = $rowAfetada["NRORDEM"];
                $nrordemRow = $row["NRORDEM"];
                
                $row["NRORDEM"] = $nrordemRowAfetada;
                $rowAfetada["NRORDEM"] = $nrordemRow;
                
                $rowAfetada["__is_new"] = false;
                $rowAfetada["changed"] = true;
                array_push($rowsToUpdate, $row, $rowAfetada);
            } else {
                array_push($rowsToUpdate, $row);
            }
            
        }
        
        
        if($this->hasTipoPrato($row)){
            if($this->isValid($row, $response)){
                if(sizeof($rowsToUpdate)>0) {
                    $this->updateConfigCardBase($rowsToUpdate);
                    $response->addDataSet(new DataSet("columnsToReturn", array()));
                } else {
                    $this->salvaConfigCardBase($request);
                    $response->addDataSet(new DataSet("configCardBase", array()));
                    $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
                }
            }
        } else {
            $response->addMessage(new Message('TCO01002_SEM_GRUPO_PRATO'));
        }
    }
    
    private function hasTipoPrato ($row){
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT CDGRTIPOPR FROM TIPOPRAT WHERE CDTIPOPRAT = :CDTIPOPRAT";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "CDTIPOPRAT" => $row["CDTIPOPRAT"]
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return !empty($result["CDGRTIPOPR"]);
    }
    
    private function retornaProximaOrdem($row) {
        $connection = $this->entityManager->getConnection();
        
        $sql = "SELECT NVL(MAX(NRORDEM)+1, 0) AS NEXTORDEM
                  FROM CONFIGCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                   AND NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE" => $row["NRCARDAPIOBASE"],
            "NRORG"          => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result["NEXTORDEM"];
    }
    
    private function retornaConfigAfetada($row, $nrordem) {
        $connection = $this->entityManager->getConnection();
        
        if($row["ORDUP"]){
            
            $sql = "SELECT NRCONFIGCARDBASE,
                           NRCARDAPIOBASE,
                           CDTIPOPRAT,
                           DSCOMPLEMENTO,
                           IDCLASSIFICACAO,
                           NRORDEM,
                           IDTEMPERATURA,
                           NRORG,
                           IDATIVO
                      FROM CONFIGCARDBASE
                     WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                       AND NRORDEM < :NRORDEM
                       AND NRORG = :NRORG
                     ORDER BY NRORDEM DESC";
                     
        }else if($row["ORDDW"]){
            
            $sql = "SELECT NRCONFIGCARDBASE,
                           NRCARDAPIOBASE,
                           CDTIPOPRAT,
                           DSCOMPLEMENTO,
                           IDCLASSIFICACAO,
                           NRORDEM,
                           IDTEMPERATURA,
                           NRORG,
                           IDATIVO
                      FROM CONFIGCARDBASE
                     WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                       AND NRORDEM > :NRORDEM
                       AND NRORG = :NRORG
                     ORDER BY NRORDEM ASC";
                     
        }
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE" => $row["NRCARDAPIOBASE"],
            "NRORDEM"        => $nrordem,
            "NRORG"          => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function isValid($row, $response) {
        $arrayErrors = array();
        
        if(strlen(iconv("UTF-8", "ISO-8859-1", $row["DSCOMPLEMENTO"])) > 50){
            $response->addMessage(new Message('TAMANHO_NOME'));
            return false;
        }
        
        return true;
    }
    
    private function salvaConfigCardBase($request) {
        $rows = $this->crudService->getRowsToSave($request, "configcardbase");
        $this->crudService->save("configcardbase", $rows);
        $rows[0]["__is_new"] = false;
        $this->crudService->addColumnToReturn($rows[0]);
    }
    
    private function updateConfigCardBase($rowsToUpdate) {
        $dataSet = new DataSet("configcardbase", $rowsToUpdate);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "configcardbase");
        $this->crudService->save("configcardbase", $rows);
    }
    
    public function removeConfigCardapioBase(DTO\Request $request, DTO\Response $response) {
        $rows = $request->getDataset()->getRows();
        $connection = $this->entityManager->getConnection();
        $configCardapioBaseEmUso = $this->configCardapioBaseEmUso($rows);

        $declaracoes = array(
            "ITEMADESAOCBASE" =>       $this->DELETE_ITEMADESAOCBASE, 
            "ITPARTSEGCARDBASE" =>     $this->DELETE_ITPARTSEGCARDBASE, 
            "DESCARTCARDBASE" =>       $this->DELETE_DESCARTCARDBASE, 
            "LIMPEZACARDBASE" =>       $this->DELETE_LIMPEZACARDBASE, 
            "AMOSTRACARDBASEQUENTE" => $this->UPDATE_AMOSTRACARDBASEQUENTE, 
            "AMOSTRACARDBASEFRIA" =>   $this->UPDATE_AMOSTRACARDBASEFRIA,
            "CONFIGCARDBASE" =>        $this->DELETE_CONFIGCARDBASE 
        );

        if(empty($configCardapioBaseEmUso)){
            try {
                $this->entityManager->beginTransaction();
                
                foreach($rows as $row) {
                    
                    $procDelPratos = new StoredProcedure($connection, 'DELETA_PRATOS_CARDBASE');
                    
                    $row['DSSEMANA'] = "T";
                    $row['NMTIPOPRAT'] = strval($row["NRCONFIGCARDBASE"]);
                    
                    $procDelPratos->addParam(new Param('P_NRCARDAPIOBASE', Param::PARAM_INPUT, $row['NRCARDAPIOBASE'],   Param::PARAM_TYPE_STR));
                    $procDelPratos->addParam(new Param('P_DIAS',         Param::PARAM_INPUT, $row['DSSEMANA'],   Param::PARAM_TYPE_STR));
                    $procDelPratos->addParam(new Param('P_CONFIGURACAO', Param::PARAM_INPUT, $row['NMTIPOPRAT'], Param::PARAM_TYPE_STR));
                    
                    $result = $procDelPratos->execute();
                    
                    foreach($declaracoes as $tabela => $declaracao) {
                        
                        $params = array(
                            "NRCONFIGCARDBASE" => $row["NRCONFIGCARDBASE"],
                            "NRORG" => $this->environment->getNrOrgTrab()
                        );
                        
                        $sql = $connection->prepare($declaracao);
                        
                        $sql->execute($params);
                        
                    }
                    
                }
            
            $response->addDataSet(new DataSet("configCardapioBaseEmUso", array()));
            $this->entityManager->commit();
            
            } catch(\Exception $e) {
                $this->entityManager->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 500, $e->getTraceAsString()));
            }
        } else {
            $response->addDataSet(new DataSet("configCardapioBaseEmUso", $configCardapioBaseEmUso));
        }
    }
    
    private function configCardapioBaseEmUso($rows) {
        $configCardapioBaseEmUso = array();
        
        foreach($rows as $row){
            $ocorrencias = $this->retornaConfigCardapioBaseEmUso($row);
            if(!empty($ocorrencias)){
                foreach($ocorrencias as $ocorrencia){
                    array_push($configCardapioBaseEmUso, $ocorrencia);
                }
            }
        }
        return $configCardapioBaseEmUso;
    }
    
    private function retornaConfigCardapioBaseEmUso($row) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT MESSAGECODE,
                       PARAM1,
                       PARAM2,
                       PARAM3
                  FROM ( /* 'Configuração em uso no Cardápio Complementar __PARAM1__.'*/
                        SELECT 'TCO01000_CONFIGURACAO_CARDAPIO_BASE_CNFCCOMPLCARDBASE_EM_USO' AS MESSAGECODE,
                               CARDAPIOBASE.CDCARDAPIOBASE AS PARAM1,
                               NULL AS PARAM2,
                               NULL AS PARAM3
                          FROM CNFCCOMPLCARDBASE
                         INNER JOIN CARDCOMPLCARDBASE
                            ON CARDCOMPLCARDBASE.NRCARDCOMPLCARDBASE = CNFCCOMPLCARDBASE.NRCARDCOMPLCARDBASE
                         INNER JOIN CARDAPIOBASE 
                            ON CARDAPIOBASE.NRCARDAPIOBASE = CARDCOMPLCARDBASE.NRCARDAPIOBASE 
                         WHERE CNFCCOMPLCARDBASE.NRCONFIGCARDBASE = :NRCONFIGCARDBASE 
                           AND CARDCOMPLCARDBASE.NRCARDAPIOBASECOMPL = :NRCARDAPIOBASE
                            
                         UNION ALL
                         
                        /* 'Configuração em uso na Proposta Comercial __PARAM1__ do Serviço __param2__ versão __PARAM3__.'*/
                        SELECT 'TCO01000_CONFIGURACAO_CARDAPIO_BASE_CONFIGSERVPROPCOM_EM_USO' AS MESSAGECODE,
                               PROPOSTACOMERCIAL.CDPROPOSTACOMERCIAL AS PARAM1,
                               SERVICOPROPOSTA.CDSERVICOPROPOSTA || ' - ' || SERVICOPROPOSTA.NMSERVICOPROPOSTA AS PARAM2,
                               VERSAOPROPCOM.CDVERSAOPROPCOM AS PARAM3
                          FROM CONFIGSERVPROPCOM
                         INNER JOIN DETALHESERVPROPCOM
                            ON DETALHESERVPROPCOM.NRDETALHESERVPROPCOM = CONFIGSERVPROPCOM.NRDETALHESERVPROPCOM
                         INNER JOIN SERVICOPROPCOM
                            ON SERVICOPROPCOM.NRSERVICOPROPCOM = DETALHESERVPROPCOM.NRSERVICOPROPCOM
                         INNER JOIN VERSAOPROPCOM
                            ON VERSAOPROPCOM.NRVERSAOPROPCOM = SERVICOPROPCOM.NRVERSAOPROPCOM
                         INNER JOIN PROPOSTACOMERCIAL
                            ON PROPOSTACOMERCIAL.NRPROPOSTACOMERCIAL = VERSAOPROPCOM.NRPROPOSTACOMERCIAL
                         INNER JOIN SERVICOPROPOSTA
                            ON SERVICOPROPOSTA.NRSERVICOPROPOSTA = SERVICOPROPCOM.NRSERVICOPROPOSTA
                         WHERE CONFIGSERVPROPCOM.NRCONFIGCARDBASE = :NRCONFIGCARDBASE
                           AND SERVICOPROPCOM.NRCARDAPIOBASE = :NRCARDAPIOBASE)";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCONFIGCARDBASE"  => $row['NRCONFIGCARDBASE'],
            "NRCARDAPIOBASE"  => $row['NRCARDAPIOBASE']
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    private function removeConfigCardBase($request, $response) {
        $rows = $this->crudService->getRowsToDelete($request, "configcardbase");
        $dataSet = new DataSet("configcardbase", $rows);
		$this->crudService->delete($response, $dataSet);
    }

}
