<?php
namespace Teknisa\TCO\Util;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;

class Mensagens extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $entityManager;
    protected $connection;
    protected $crudService;
    protected $parameterbag;
    protected $newCode;
    protected $dataSourceManager;
    
    public function __construct($crudService, $parameterbag, $environment, $connection, $entityManager, $newCode, $dataSourceManager) {
        $this->crudService       = $crudService;
        $this->parameterbag      = $parameterbag;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->entityManager     = $entityManager;
        $this->newCode           = $newCode;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    /*
        Caso seja apenas uma mensagem, monte:
        $row = array(
            'TIPO' => 'MENSAGEM',
            'MSG' => 'conteudo_da_mensagem'
        );
        
        Caso seja uma lista de mensagens:
        $row = array(
            'TIPO' => 'LISTA'
            'CDFILIAL' => 'XXXX',
            'NRLISTA'  => 'XXXXXX'
        );
    
    */
    public function convertMessage($row){
        $resultado = array();
        if($row['TIPO'] === 'MENSAGEM'){
            $resultado = $this->converter($row['MSG']);
            $resultado = array($resultado);
            
        }else if($row['TIPO'] === 'LISTA'){
            $erros = $this->getErros($row['NRLISTA'], $row['CDFILIAL']);
            foreach($erros as $erro){
                array_push($resultado, $this->converter($erro['DSERRO']));
            }
        }
        
        return $resultado;
    }
    
    //converte a mensagem retornando um array formatado com os parametros necessários ou uma string com o erro puro
    public function converter($message){
		$messageUpper 	= strtoupper($message);
		$codeStart	 	= strpos($messageUpper, "_MSGSTA_");
		$codeEnd		= strpos($messageUpper, "_MSGEND_");
		
		if ($codeStart !== FALSE && $codeEnd !== FALSE){
			$msgStart = $codeStart + strlen("_MSGSTA_");
			$msgEnd = $codeEnd - $msgStart;

	        $paramstring = substr($message, $msgStart, $msgEnd);
	        $params = explode("$$", $paramstring);
	        
            $mensagem = array_shift($params);
	        
	        $resultado = array("MSGCODE"=>$mensagem);
	        $resultado['PARAMS'] = array();
	        
	        if(sizeof($params) > 0){
	            $parametros = array();
	            foreach ($params as $index => $value) {
	                 $parametros['PARAM'.$index] = $value;
	            }
	            $resultado['PARAMS'] = $parametros;
	        }
	        
	        return $resultado;
		}
		return $message;
	}
	
	//se for gerada uma lista de erros, recebe nrlista e cdfilial e retorna os erros
	public function getErros($nrLista, $cdFilial){
	    try{
    	    $sql = "SELECT DISTINCT DSERRO FROM LISTAERROS WHERE NRLISTA = :NRLISTA AND CDFILIAL = :CDFILIAL ORDER BY DSERRO";
    	    
    	    $query = $this->connection->prepare($sql);
            
            $params = array(
                'NRLISTA' => $nrLista,
                'CDFILIAL'=> $cdFilial
            ); 
            $query->execute($params);
            
            $result = $query->fetchAll(\PDO::FETCH_ASSOC);
            
            return $result;
	    } catch(\Exception $e){
            return array(
                'MSGCODE' => '', 
                'PARAMS' => array()
            );
        }
	}
}